# This program is a part of Ruby/FLTK samples.
# Author:: Takaaki Tateishi <mailto:ttate@kt.jaist.ac.jp>
# Copyright:: Copyright (c) 2002 Takaaki Tateishi
# License:: LGPL

require 'fltk'
require 'fltk/thread'
require 'mkmf'

RUBY = CONFIG["prefix"] + "/bin/" + CONFIG["ruby_install_name"]

if( ! defined?(SAMPLE_SCRIPTS) )
  SAMPLE_SCRIPTS = [
    ["Sample1", "sample1.rb"],
    ["Sample2", "sample2.rb"],
    ["Sample3", "sample3.rb"],
    ["Sample4", "sample4.rb"],
    ["Sample5", "sample5.rb"],
    ["Sample6", "sample6.rb"],
    ["Sample7", "sample7.rb"],
    ["Sample8", "sample8.rb"],
    ["Sample9", "sample9.rb"],
    ["ListBox", "listbox.rb"],
    ["Resizing","resize.rb"],
    ["Editor",  "editor.rb"],
    ["Othello", "othello.rb"],
    ["Clock",   "clock.rb"],
    ["Images",  "image.rb"],
    ["Wizard",  "wizard.rb"],
    ["Class Browser", "cbrowser.rb"],
  ]
end
if( ! defined?(BUTTON_MESSAGE) )
  BUTTON_MESSAGE = "Click to execute"
end
if( ! defined?(FOOTER_MESSAGE) )
  FOOTER_MESSAGE = [
    "Copyright (C) 2002 Takaaki Tateishi",
    "This program is a sample for Ruby/FLTK.",
  ].join("\n")
end

class DemoButton < FLTK::Button

  # Highlight the button when the mouse pointer is on the button.
  def handle(e)
    font = labelfont
    case e
    when FLTK::ENTER
      color(FLTK::CYAN)
      if( font & FLTK::BOLD == 0 )
	labelfont(font + FLTK::BOLD)
      end
      redraw
    when FLTK::LEAVE
      color(FLTK::GRAY)
      if( font & FLTK::BOLD == FLTK::BOLD )
	labelfont(font - FLTK::BOLD)
      end
      redraw
    end
    return super(e)
  end
end

class DemoWindow < FLTK::Window
  LEFT_BORDER_WIDTH  = 10
  RIGHT_BORDER_WIDTH = 10
  TITLE_HEIGHT = 20
  TITLE = "Ruby/FLTK Demo"
  MSG_HEIGHT = 100
  MSG_BORDER = 10

  def initialize(w, h, label=TITLE)
    super(w,h,label){|win|
      FLTK::Box.new(w, TITLE_HEIGHT, TITLE)

      scroll_x = 0
      scroll_y = TITLE_HEIGHT
      scroll_w = w
      scroll_h = h - TITLE_HEIGHT - MSG_HEIGHT - MSG_BORDER
      FLTK::Scroll.new(scroll_x, scroll_y, scroll_w, scroll_h){|scroll|
	scroll.scrolltype = FLTK::Scroll::VERTICAL

	pack_x = LEFT_BORDER_WIDTH
	pack_y = TITLE_HEIGHT
	pack_w = w - LEFT_BORDER_WIDTH - RIGHT_BORDER_WIDTH
	pack_h = h - TITLE_HEIGHT - MSG_HEIGHT - MSG_BORDER
	FLTK::Pack.new(pack_x, pack_y, pack_w, pack_h){|pack|
	  SAMPLE_SCRIPTS.each{|name,script|
	    button = DemoButton.new(pack.x, 0, pack.w, 25, name){|button,data|
	      # Here is a definition of the callback.
	      path = File.join(File.dirname($0), script)
	      Thread.new{
	        system("#{RUBY} #{path}")
	      }
	    }
	    # If you use fltk-1.1.x, Widget#tooltip is available.
	    if( defined?(button.tooltip) )
	      button.tooltip = BUTTON_MESSAGE
	    end
	  }
	  DemoButton.new(pack.x, 0, pack.w, 25, "Quit"){ exit(0) }
	} # end of Pack
	win.resizable(scroll)
	p win.resizable
      } # end of Scroll

      out_x = LEFT_BORDER_WIDTH
      out_y = h - MSG_HEIGHT - MSG_BORDER
      out_w = w - LEFT_BORDER_WIDTH - RIGHT_BORDER_WIDTH
      out_h = MSG_HEIGHT
      out = FLTK::MultilineOutput.new(out_x, out_y, out_w, out_h)
      out.value = FOOTER_MESSAGE

      # If the "new" method takes a block, the block should be evaluated here.
      if( block_given? )
	yield(win)
      end
    }
  end
end

DemoWindow.new(400,500){|win|
  win.show
}

GC.start
FLTK.run
