# This program is a part of the Ruby/FLTK tests.
# Author:: Jeremy Henty <mailto:jeremy_henty@users.sourceforge.net>
# Copyright:: Copyright (c) 2005 Jeremy Henty 
# License:: LGPL

=begin

Show the available colors.

=end

require 'fltk'

name_this        = Hash.new
color_this       = Hash.new
color_complement = Hash.new
name_darker      = Hash.new
color_darker     = Hash.new

# hash the values of the colors, their complements, and their darker
# relatives

[
  [ "BLACK", "WHITE",   ], 
  [ "RED",   "CYAN",    ], 
  [ "GREEN", "MAGENTA", ], 
  [ "BLUE",  "YELLOW",  ], 
].each do |colors|
  colors.each do |c|
    darker = "DARK_%s" % [ c, ]
    name_this[c] = c
    name_darker[c] = darker
    begin
      color_darker[c] = 
        FLTK.const_get(darker)
    rescue NameError # because DARK_WHITE and DARK_BLACK don't exist 
    end
  end
  c0, c1, = *colors
  color0 = FLTK.const_get(c0)
  color1 = FLTK.const_get(c1)
  color_this[c0] = color0
  color_complement[c0] = color1
  color_this[c1] = color1
  color_complement[c1] = color0
end

color_matrix = [
  # the first element means "do we show the dark colors too?" 
  [ false, "BLACK", ], 
  [ true,  "RED",   "GREEN",   "BLUE",   ], 
  [ true,  "CYAN",  "MAGENTA", "YELLOW", ], 
  [ false, "WHITE", ], 
]

# geometry 
h =  30 # height of a row 
w = 150 # width of a column 
column_count = 3 # (this must play nicely with "color_matrix") 
row_count = 0
color_matrix.each do |eh,|
  row_count += eh ? 2 : 1
end

# create the GUI 
FLTK::Window.new(column_count*w,row_count*h,"Colors"){|win|
  y = 0
  color_matrix.each{|eh,*colors|
    dx = (column_count/colors.size)*w
    proc{|do_colors|
      do_colors.call(colors,color_this,name_this)
      if eh
        do_colors.call(colors,color_darker,name_darker)
      end
    }.call proc{|_colors,color_hash,name_hash,|
      x = 0
      _colors.each{|color|
        box = FLTK::Box.new(x,y,dx,h)
        box.box(FLTK::FLAT_BOX)
        box.label(name_hash[color])
        box.color(color_hash[color])
        box.labelcolor(color_complement[color])
        x += dx
      }
      y += h
    }
  }
  win.show
}

FLTK.run
