require 'opengl'
require 'fltk'

class ShapeWindow < FLTK::GLWindow
  attr_accessor :sides, :overlay_sides

  def initialize(*args)
    super(*args)
    @sides = 3
    @overlay_sides = 3
  end

  def draw
    if( !valid? )
      valid = true
      GL.LoadIdentity
      GL.Viewport(0,0,w,h)
    end
    GL.Clear(GL::COLOR_BUFFER_BIT)
    GL.Begin(GL::POLYGON)
    for i in 0 .. @sides
      ang = (i * 2 * Math::PI) / @sides
      GL.Color3f(i / @sides, i / @sides, i / @sides)
      GL.Vertex3f(Math.cos(ang), Math.sin(ang), 0)
    end
    GL.End
  end

  def draw_overlay
    if( ! valid? )
      valid = true
      GL.LoadIdentity;
      GL.Viewport(0,0,w,h);
    end
    FLTK::GL.color(FLTK::RED);
    GL.Begin(GL::LINE_LOOP);
    for i in 0 .. @overlay_sides
      ang = (i * 2 * Math::PI) / @overlay_sides;
      GL.Vertex3f(Math.cos(ang), Math.sin(ang), 0);
    end
    GL.End;
  end
end

$win = FLTK::Window.new(300,370){|win|
  $sw = ShapeWindow.popup(10,75,win.w-20,win.h-90){|sw|
    # sw.mode = FL::RGB
  }
  win.resizable($sw)

  slider = FLTK::HorSlider.new(60,5,win.w-70,30,"Sides:")
  slider.align(FLTK::ALIGN_LEFT)
  slider.callback($sw){|slider,sw|
    sw.sides = slider.value
    sw.redraw
  }
  slider.value($sw.sides)
  slider.step = 1
  slider.bounds(3,40)

  oslider = FLTK::HorSlider.new(60,40,win.w-70,30,"Overlay:")
  oslider.align(FLTK::ALIGN_LEFT)
  oslider.callback($sw){|slider,sw|
    sw.overlay_sides = slider.value
    sw.redraw_overlay
  }
  oslider.value($sw.overlay_sides)
  oslider.step = 1
  oslider.bounds(3,40)
}
$win.show
$sw.can_do_overlay
$sw.show
$sw.redraw_overlay
FLTK.run
