require 'fltk'
require 'flvw'

SZ = 5

class VTableChild < FLTK::VTable
  include FLTK::Drawable
    def initialize( px, py, pw, ph, l='' )
	super(px,py,pw,ph,l)
	@lw = -1;
	@cw = []
    end

    def draw_cell(offset, px, py, pw, ph, pr, pc)
	style = FLTK::VStyle.new
	get_style(style,pr,pc)
	super
	if (pc == 1) && (pr > -1)
	    if get_value(pr,pc)[0] == ?M
		#draw_symbol('@circle', px-offset+(pw-SZ)/2, py+(ph-SZ)/2,SZ,SZ, 32+16)
		FLTK.draw('@circle', px-offset-8,py-4,pw-2,ph, style.align)
	    end
	else
#	    draw_text(get_value(pr,pc), px-offset,py-8,pw-4,ph, style.align)
	    draw_text(get_value(pr,pc), px-offset-8,py-8,pw-4,ph, style.align)
	end
    end

#//	Note: This is so flexible, you don't *have* to use style
#//	if you'd rather program the conditions...
    def get_style(s,pr,pc)
	super
	if (pr<0)				#//	Heading/Footing is bold
		s.font( s.font + FLTK::BOLD )
	end
	if (pr==-2)				#//	Row footer exception
		s.background( FLTK::BLACK )	#//	Black background
		s.foreground( FLTK::WHITE )	#//	White text
		s.frame( FLTK::FLAT_BOX )	#//	No box
		s.align(FLTK::ALIGN_RIGHT)	#//	Right aligned
	end
	if (pr>-1 && pc>-1 && (pr%2)==0 && pc<4 )	#//	Highlight every other row
		s.background( 32+21 )
	end
	st = get_value(pr,pc)
	if (st)
	    if st.index("-$") 		#//	Negative $ are RED
  		s.border(FLTK::VB_OUTER_ALL)	#//	Nice dark box
		s.foreground(FLTK::RED)	#//	Text in red
		s.background(215) 		#//	Pale yellow background
	    end
	end
    end

    def col_width(pc,pr=0)
	#//	Another way would be to override handle for FL_SIZE.  We could also
	#//	spend a lot of time setting styles for the columns and returning
	#//	that, but since we're calculating it anyway, why bother.

	scrollbar_width = (scrollbar.visible ? scrollbar.w : 0)
	vw = w- scrollbar_width-1

	#//	Either always calculate or be sure to check that width
	#//	hasn't changed.
	if (vw!=@lw)				#//	Width change, recalculate
		@cw[0] = (vw*30)/100;		#//	30		Name
		@cw[1] = (vw*15)/100;		#//	10		Gender
		@cw[2] = (vw*20)/100;		#//	 5%		Phone
		@cw[3] = (vw*20)/100;		#//	15%		Date
		ww = 0
		(0..3).each {|t|
		    ww += @cw[t]
		}
		@cw[4] = vw-ww-1			#//	~30% +/- previous rounding errors
		@lw = vw
	end
	@cw[pc]
    end
end


$val =
[
	[ "",									"",				"",					"Total",		"$213.23"	],
	[ "Name",							"Male",		"Phone",		"Meet",				"Due"	],
	[	"Smith, John", 			"Male", 	"555-1212", "05/14/1999", "$50.00" ],
	[	"Jet, Joan", 				"Female", "234-1234", "12/22/1999", "$25.75" ],
	[	"Frampton, Peter", 	"Male", 	"324-8723", "09/17/1999", "$35.22" ],
	[	"Ozborne, Ozzy", 		"Male", 	"545-8273", "04/05/1999", "$84.72" ],
	[	"Person, Test", 		"Male", 	"563-2938", "08/04/1999", "$38.83" ],
	[	"Clairborne, Liz", 	"Female", "723-9382", "03/17/1999", "-$61.34" ],
	[	"Curie , Madam", 		"Female", "234-7382", "02/23/1999", "$22.43" ],
	[	"Benetar, Pat", 		"Female", "121-2837", "06/09/1999", "$14.18" ],
	[	"Krull, Diane", 		"Female", "345-2384", "01/11/2000", "-$68.88" ],
	[	"Simpson, Bart", 		"Male", 	"928-1342", "07/23/1999", "$72.32" ]
]

def get_value(r, c )
    $val[r+2][c]
end


w = FLTK::Window.new( 432, 255, "Dynamic row size example" )
l = VTableChild.new( 10, 10, 412, 235, "Current Appointments" )
w.end

w.resizable(l)
#  //	row height (17) * rows (10) +
#  //	row height headers (17+4) * number of headers (3) +
#  //	Top/Bottom box margins (2)

#	//	Comment/or uncomment any of these lines to see the result
l.rows(10)
l.cols(5)
l.has_scrollbar(FLTK::VS_VERTICAL)
l.feature(FLTK::VF_ROW_ENDS|FLTK::VF_ROW_SELECT)
l.global_style.align(FLTK::ALIGN_LEFT)	#//	Left alignment
l.global_style.font_size(20)
l.global_style.height(l.global_style.font_size*1.2)

l.col_style[4].border_color(FLTK::BLACK)	#//	Black line
l.col_style[4].border(FLTK::VB_LEFT)		#//	On left of last column
l.col_style[4].background(93)			#//	Orange background? :)

#	//	Have to align something differently! :)
l.col_style[1].align(FLTK::ALIGN_CENTER)		#//	Gender
l.col_style[4].align(FLTK::ALIGN_RIGHT)		#//	Amount

#	//	We have a created 3 styles at this point, besides of course
#	//	global_style, row_style, col_style, and the cell_style in our
#	//	one row_style.

w.show

FLTK.run
